package pwc.taxtech.atms.controller;

import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import pwc.taxtech.atms.common.ApplicationException;
import pwc.taxtech.atms.common.ftp.FTPClientPool;
import pwc.taxtech.atms.dto.*;
import pwc.taxtech.atms.entitiy.Template;
import pwc.taxtech.atms.service.TemplateService;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URISyntaxException;
import java.util.Collections;
import java.util.List;

@RestController
@RequestMapping(value = "api/v1/template")
public class TemplateController {
    private static final Logger logger = LoggerFactory.getLogger(TemplateController.class);

    @Autowired
    TemplateService templateService;

    @Autowired
    FTPClientPool ftpClientPool;

    @RequestMapping(value = "get", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public @ResponseBody
    List<TemplateDto> get(@RequestParam String templateGroupID, @RequestParam String reportType) {
        if (StringUtils.isEmpty(templateGroupID)) {
            return Collections.emptyList();
        }

        try {
            return templateService.get(templateGroupID, (reportType != null && !"null".equals(reportType)) ? Integer.parseInt(reportType) : null);
        } catch (Exception e) {
            logger.error("GetCellConfigList", e);
        }
        return Collections.emptyList();
    }

    @RequestMapping(value = "getTemplateJson", method = RequestMethod.POST, produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    public @ResponseBody
    void getTemplateBlob(@RequestParam String templateID, HttpServletResponse response) throws URISyntaxException {
        String filePath;
        File templateFile;
        InputStream inputStream = null;
        Template template = templateService.getTemplateByID(templateID);
        String templatePath = templateService.getTemplatePath(templateID);
        if (StringUtils.isBlank(templateID)) {
            return;
        }

        if (StringUtils.isBlank(templatePath)) {
            return;
        }
        filePath = this.getClass().getResource("").toURI().getPath();
        String tempPath = filePath.substring(0, filePath.indexOf("classes") + "\\classes".length());
        templateFile = new File(tempPath + templatePath);

        try {
            //如果是系统报表就取本地文件夹，如果不是就取FTP
            if (template.getIsSystemType()) {
                inputStream = new BufferedInputStream(new FileInputStream(templateFile));
            } else {
                inputStream = ftpClientPool.download(templatePath);
            }
            //客户端保存的文件名
            String customFileName = "template_" + DateTime.now().toString("yyyyMMddHHmmss") + ".xlsx";
            response.setHeader("Content-Disposition", String.format("inline; filename=\"" + customFileName + "\""));
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            int len = 0;
            byte[] buffer = new byte[1024];
            OutputStream out = response.getOutputStream();
            while ((len = inputStream.read(buffer)) > 0) {
                out.write(buffer, 0, len);
            }

            //FileCopyUtils.copy(inputStream, response.getOutputStream());
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (Exception e) {
            logger.error("Error downloading template file template.xlsx", e);
            throw new ApplicationException("Error downloading template file template.xlsx", e);
        } finally {
            try {
                templateFile = null;
                if (inputStream != null) {
                    inputStream.close();
                }
            } catch (Exception e) {
                logger.error("Error closing inputStream. ", e);
            } finally {
                inputStream = null;
            }
        }
    }

    @RequestMapping(value = "getTemplateUniqList", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public @ResponseBody
    List<TemplateUniqDto> getTemplateUniqList(String serviceTypeID, Integer payTaxType, Integer reportType, String industryIDs) {
        return templateService.getTemplateUniqList(serviceTypeID, payTaxType, reportType, industryIDs);
    }

    @RequestMapping(value = "updateTemplateName", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public @ResponseBody
    OperationResultDto<Object> updateTemplateName(@RequestBody UpateNameParam param) {
        return templateService.updateTemplateName(param);
    }

    @RequestMapping(value = "deleteTemplate", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public @ResponseBody
    OperationResultDto<String> deleteTemplate(@RequestBody DeleteTemplateParam param) {
        OperationResultDto<String> result = templateService.deleteTemplate(param);
        if (result.getResult() && StringUtils.isNotBlank(result.getData())) {
            try {
                ftpClientPool.delete(result.getData());
            } catch (Exception e) {
            }
        }
        return result;
    }

    @RequestMapping(value = "rowColName/{id}", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public @ResponseBody
    OperationResultDto setRowColName(@PathVariable String id, @RequestBody List<CellBriefDto> cellInfo) {
        return templateService.setRowColName(id, cellInfo);
    }
}
