package pwc.taxtech.atms.service.impl;

import org.apache.commons.lang3.StringUtils;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.constant.enums.CellDataSourceType;
import pwc.taxtech.atms.dto.CellTemplateConfigDto;
import pwc.taxtech.atms.entitiy.CellTemplate;
import pwc.taxtech.atms.entitiy.CellTemplateConfig;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;

public final class CellConfigTranslater {

    public static CellTemplateConfigDto getConfigDto(CellTemplate template, List<CellTemplateConfig> configList) {
        if (template == null) {
            return null;
        }

        return getConfigDto(template.getID(), template.getReportTemplateID(), template.getRowIndex(), template.getRowName(), template.getColumnIndex(),
                template.getColumnName(), template.getDataType(), template.getIsReadOnly(), template.getComment(), configList);
    }

    private static CellTemplateConfigDto getConfigDto(String configID, String templateID, int rowIndex, String rowName, int columnIndex,
                                                      String columnName, Integer dataType, Integer isReadOnly, String description, List<CellTemplateConfig> configList) {
        if (configList == null) {
            return null;
        }

        CellTemplateConfigDto cellTemplateConfigDto = new CellTemplateConfigDto();
        cellTemplateConfigDto.setCellTemplateID(configID);
        cellTemplateConfigDto.setTemplateID(templateID);
        cellTemplateConfigDto.setRowIndex(rowIndex);
        cellTemplateConfigDto.setRowName(rowName);
        cellTemplateConfigDto.setColumnIndex(columnIndex);
        cellTemplateConfigDto.setColumnName(columnName);
        cellTemplateConfigDto.setDataType(dataType);
        cellTemplateConfigDto.setIsReadOnly(isReadOnly);
        cellTemplateConfigDto.setFormulaDescription(description);

        Optional<CellTemplateConfig> formulaItem = configList.stream().filter(a -> a.getDataSourceType().equals(CellDataSourceType.Formula.getCode())).findFirst();
        if (formulaItem.isPresent()) {
            cellTemplateConfigDto.setHasFormula(true);
            cellTemplateConfigDto.setFormula(formulaItem.get().getFormula());
            if (StringUtils.isEmpty(cellTemplateConfigDto.getFormulaDescription())
                    && !StringUtils.isEmpty(formulaItem.get().getFormulaDescription())) {
                cellTemplateConfigDto.setFormulaDescription(formulaItem.get().getFormulaDescription());
            }
            cellTemplateConfigDto.setFormulaDataSource(formulaItem.get().getFormulaDataSource());
            cellTemplateConfigDto.setCellTemplateID(formulaItem.get().getCellTemplateID());
        }

        Optional<CellTemplateConfig> voucherItem = configList.stream().filter(x -> x.getDataSourceType().equals(CellDataSourceType.Voucher.getCode())).findFirst();
        if (voucherItem.isPresent()) {
            cellTemplateConfigDto.setHasVoucher(true);
            cellTemplateConfigDto.setVoucherKeyword(voucherItem.get().getVoucherKeyword() == null ? "" : voucherItem.get().getVoucherKeyword());
            if (!StringUtils.isEmpty(voucherItem.get().getAccountCodes())) {
                cellTemplateConfigDto.setAccountCodes(getList(voucherItem.get().getAccountCodes()));
            }
        }

        Optional<CellTemplateConfig> invoiceItem = configList.stream().filter(x -> x.getDataSourceType().equals(CellDataSourceType.OutputInvoice.getCode()) || x.getDataSourceType().equals(CellDataSourceType.InputInvoice.getCode()) || x.getDataSourceType().equals(CellDataSourceType.CustomInvoice.getCode())).findFirst();
        if (invoiceItem.isPresent()) {
            cellTemplateConfigDto.setHasInvoice(true);
            cellTemplateConfigDto.setInvoiceType(invoiceItem.get().getInvoiceType());
            cellTemplateConfigDto.setInvoiceAmountType(invoiceItem.get().getInvoiceAmountType());
            if (!StringUtils.isEmpty(invoiceItem.get().getTaxRate())) {
                cellTemplateConfigDto.setTaxRate(getList(invoiceItem.get().getTaxRate()));
            }

            if (!StringUtils.isEmpty(invoiceItem.get().getInvoiceCategory())) {
                List<String> invoiceCategoryStrs;
                invoiceCategoryStrs = getList(invoiceItem.get().getInvoiceCategory());
                List<Integer> ints = new ArrayList<>();
                assert invoiceCategoryStrs != null;
                for (String categoryStr : invoiceCategoryStrs) {
                    int categoryVal;
                    categoryVal = Integer.parseInt(categoryStr);
                    ints.add(categoryVal);
                }
                cellTemplateConfigDto.setInvoiceCategory(ints);
            }
        }

        Optional<CellTemplateConfig> keyInItem = configList.stream().filter(x -> x.getDataSourceType().equals(CellDataSourceType.KeyIn.getCode())).findFirst();
        if (keyInItem.isPresent()) {
            cellTemplateConfigDto.setHasKeyIn(true);
        }

        Optional<CellTemplateConfig> modelItem = configList.stream().filter(x -> x.getDataSourceType().equals(CellDataSourceType.RelatedModel.getCode())).findFirst();
        if (modelItem.isPresent()) {
            cellTemplateConfigDto.setHasModel(true);
            cellTemplateConfigDto.setModelIDs(getList(modelItem.get().getModelIDs()));
        }

        Optional<CellTemplateConfig> validationItem = configList.stream().filter(x -> x.getDataSourceType().equals(CellDataSourceType.Validation.getCode())).findFirst();
        if (validationItem.isPresent()) {
            cellTemplateConfigDto.setHasValidation(true);
            cellTemplateConfigDto.setValidation(validationItem.get().getValidation());
            cellTemplateConfigDto.setValidationDescription(validationItem.get().getValidationDescription());
            cellTemplateConfigDto.setCellTemplateID(validationItem.get().getCellTemplateID());
        }
        return cellTemplateConfigDto;
    }

    private static List<String> getList(String joinString) {
        if (StringUtils.isEmpty(joinString)) {
            return null;
        }

        return new ArrayList<>(Arrays.asList(joinString.split(Constant.Comma)));
    }
}
